/** @file counter_media.cpp
 * This file contains implementation of the counter model modiator class.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#include <counter_media.h>

#include <sync.h>

#include <interface.h>
#include <vpi_media.h>

#include <iostream>

namespace cpptesk {
namespace examples {
namespace counter {

CounterMediator::CounterMediator()
{
    SET_INPUT_ADAPTER(iface1, CounterMediator::serialize_iface1);
    SET_OUTPUT_ADAPTER(iface2, CounterMediator::deserialize_iface2);

    SET_REACTION_TIMEOUT(CounterMediator::REACTION_TIMEOUT);

    clear_inputs(&inputs);
    clear_outputs(&outputs);

    SET_ARBITER(iface2, matcher_iface2);

    CALL_OUTPUT_LISTENER(CounterMediator::listen_iface2, iface2);
    debug(DEBUG_ERROR);
}

CounterMediator::~CounterMediator() {}

bool CounterMediator::isIface1Ready() const
{
    return Counter::isIface1Ready();
}

DEFINE_PROCESS(CounterMediator::serialize_iface1)
{
    InputData msg = CAST_MESSAGE(InputData);

    START_PROCESS();

#ifdef DEBUG
	std::cout << "CounterMediator::serialize_iface1" << std::endl;
#endif

    CAPTURE_IFACE();

    inputs.inc = 1;

    RELEASE_IFACE();

    CYCLE();

    STOP_PROCESS();
}

DEFINE_REACTION(CounterMediator::deserialize_iface2)
{
    OutputData &msg = CAST_MESSAGE(OutputData);

    START_PROCESS();

#ifdef DEBUG
        std::cout << "CounterMediator::deserialize_iface2" << std::endl;
#endif

    WAIT_REACTION(true);

    msg.set_data(outputs.data);

    NEXT_REACTION();

    STOP_PROCESS();
}

void CounterMediator::initialize()
{
    ::start_testbench();
}

void CounterMediator::finalize()
{
    ::stop_testbench();
}

void CounterMediator::setInputs()
{
    ::map_state_down(&inputs);
}

void CounterMediator::getOutputs()
{
    ::map_state_up(&outputs);
}

void CounterMediator::simulate()
{
    ::wait_for_check();
}

}}}

