/** @file fifo_media.h
 *       FIFO.
 *
 * @author   (chupilko@ispras.ru)
 */

#pragma once

#include <hw/media.hpp>
#include <fifo_model.h>
#include <interface.h>

#include <string>

namespace cpptesk {
namespace examples {
namespace fifo {

/**
 * @brief    FIFO.
 */
ADAPTER(FIFOMediator, FIFO)
{
public:
    FIFOMediator();
    virtual ~FIFOMediator();

    ///      ( ).
    const static int REACTION_TIMEOUT = 100;

    virtual bool isIface1Ready() const;
    virtual bool isIface2Ready() const;

    ///    iface1.
    DECLARE_PROCESS(serialize_iface1);
    ///    iface2.
    DECLARE_PROCESS(serialize_iface2);

    ///    iface3.
    DECLARE_PROCESS(deserialize_iface3);

    ///      iface3.
    DEFINE_BASIC_OUTPUT_LISTENER(listen_iface3, iface3, outputs.DO_VAL);

    ///         .
    DECLARE_ARBITER(PRIORITY_ARBITER, matcher_iface3);

    ///   .
    virtual void initialize();

    ///   .
    virtual void finalize();

    ///     .
    virtual void setInputs();

    ///     .
    virtual void getOutputs();

    ///         .
    virtual void simulate();

private:
    ///      .
    inputs_t  inputs;

    ///      .
    outputs_t outputs;
};

}}}

