/** @file fifo_model.cpp
 *   FIFO.
 *
 * @author   (chupilko@ispras.ru)
 */

#include <fifo_model.h>

namespace cpptesk {
namespace examples {
namespace fifo {

FIFO::FIFO() : coverage(*this)
{
    ///     .
    ADD_INPUT(iface1);
    ADD_INPUT(iface2);
    ADD_OUTPUT(iface3);
}

FIFO::~FIFO() {}

bool FIFO::isIface1Ready() const
{
    ///  isReady  true,   iface1   - .
    return iface1.isReady();
}

bool FIFO::isIface2Ready() const
{
    ///  isReady  true,   iface2   - .
    return iface2.isReady();
}

DEFINE_STIMULUS(FIFO::push_msg)
{
    //     .
    InputData data = CAST_MESSAGE(InputData);

    //   FIFO.
    push_item(data.get_data());

    //       iface    .
    START_STIMULUS(PARALLEL);

    //    1  .
    CYCLE();

    //  .
    STOP_STIMULUS();
}

DEFINE_STIMULUS(FIFO::pop_msg)
{
    //     .
    InputData data = CAST_MESSAGE(InputData);

    //   .
    START_PROCESS();

    /*      iface
       ( ,     iface  data).
     */
    RECV_STIMULUS(PARALLEL, GET_IFACE(), data);

    //      ,   DUT.
    OutputData outdata = OutputData(pop_item());

    //   " ,    ".
    get_pop_msg(process, iface3, outdata);

    //  .
    STOP_PROCESS();
}

DEFINE_REACTION(FIFO::get_pop_msg)
{
    //    .
    SEND_REACTION(SEQUENTIAL, GET_IFACE(), GET_MESSAGE());
}

void FIFO::push_item(uint8_t data)
{
    //    ,     .
    assert(fifo.size() < 16);
    //     .
    fifo.push_back(data);
}

uint8_t FIFO::pop_item(void)
{
    //     ,    .
    assert(fifo.size() > 0);
    //     .
    uint8_t data = fifo[0];
    //     .
    fifo.erase(fifo.begin());
    //  .
    return data;
}

bool FIFO::is_empty(void)
{
    return fifo.size() == 0;
}

bool FIFO::is_full(void)
{
    return fifo.size() == 16;
}

int FIFO::fifo_fulness(void)
{
    return fifo.size();
}

void FIFO::refresh_coverage(void)
{
    coverage.out_cov_FF();
}

}}}

