/** @file fifo_model.h
 *     FIFO.
 *
 * @author   (chupilko@ispras.ru)
 */

#pragma once

#include <hw/model.hpp>

#include <fifo_coverage.h>
#include <fifo_msg.h>

namespace cpptesk {
namespace examples {
namespace fifo {

/**
 * @brief    FIFO.
 */
MODEL(FIFO) {
public:
    FIFO();
    virtual ~FIFO();

    ///   iface1     .
    DECLARE_INPUT(iface1);
    ///   iface2     .
    DECLARE_INPUT(iface2);
    ///   iface3    .
    DECLARE_OUTPUT(iface3);

    /// ,    iface1. @return iface1.isReady()
    virtual bool isIface1Ready() const;
    /// ,    iface2. @return iface2.isReady()
    virtual bool isIface2Ready() const;

    ///  "   ".
    DECLARE_STIMULUS(push_msg);
    ///  "    ".
    DECLARE_STIMULUS(pop_msg);
    ///  " ,   ".
    DECLARE_REACTION(get_pop_msg);

    ///      FIFO.
    void push_item(uint8_t data);
    ///      FIFO      @return    FIFO
    uint8_t pop_item(void);
    ///    FIFO   @return size() == 0
    bool is_empty(void);
    ///    FIFO   @return size() == 16
    bool is_full(void);
    ///        FIFO @return size()
    int fifo_fulness(void);

    ///  
    FIFOCoverage coverage;
    ///        NOP
    void refresh_coverage(void);
protected:
    ///   FIFO.
    std::vector<uint8_t> fifo;
};

}}}

